//
// menuposition.js - Dynamically place CSS drop-down and fly-out menus
//	when javascript is enabled
//
// Version 1.1
// Copyright 2007-2009 Brian Katzung, Kappa Computer Solutions, LLC
//
//   This program is free software; you can redistribute it and/or
//   modify it under the terms of the GNU General Public License as
//   published by the Free Software Foundation; either version 2 of
//   the License, or (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
//   MA 02111-1307 USA
//
// For more information, contact:
//
//	Kappa Computer Solutions, LLC
//	877.367.8837 (US) or 781.370.1093
//	http://www.kappacs.com/
//
// Requires prototype.js (http://www.prototypejs.org)
//	 or mootools.js (http://www.mootools.net)
// Requires viewport.js (based on http://www.quirksmode.org/viewport)
//
// 2007-09-04	Created by Brian Katzung, Kappa Computer Solutions, LLC

//======================================================================

// The minimum sub-menu overlap on the adjustable axis (left or right
// on drop-downs and raise-ups, or up and down on fly-outs) to insure
// that you can actually get the cursor from the item to the sub-menu
window.mpMinOverlap = 10

//======================================================================

//----------------------------------------------------------------------
// Get menu positioning directions from class names or menu layout
// mpDown  - horizontal menu will drop down
// mpUp    - horizontal menu will raise up
// mpRight - vertical menu will fly out right
// mpLeft  - vertical menu will fly out left

function mpGetDirection (menu, item)
{
	var direction = new Array()
	var dirEl = $(menu.mpDirEl? menu.mpDirEl: menu)

	// Explicit up or down first

	if (dirEl.hasClassName('mpDown'))
		direction[0] = 'down'
	else if (dirEl.hasClassName('mpUp'))
		direction[0] = 'up'

	// Explicit right or left (first or second)

	if (dirEl.hasClassName('mpRight'))
		direction[direction.length] = 'right'
	else if (dirEl.hasClassName('mpLeft'))
		direction[direction.length] = 'left'

	// Supplement explicit settings with defaults as needed

	if (direction.length < 2)
		switch (direction[0])
		{
		case 'up':
		case 'down':
			// Sub-menus of drop-downs will fly out right
			direction[1] = 'right'

		case 'left':
		case 'right':
			// Nothing additional needed for fly-outs
			break

		default:
			// Default heuristic
			switch ($(item).getStyle('float'))
			{
			case 'left':
			case 'right':
				// Floated items (probably a horizontal menu):
				// default to down & right
				direction[0] = 'down'

			default:
				// Non-floated items (probably a vertical menu):
				// default to right
				direction[direction.length] = 'right'
			}
			break
		}

	menu.mpDirection = direction
}

//----------------------------------------------------------------------
// Return parent menu

function mpGetParentMenu (node)
{
	while (node = node.parentNode)
		if (node.tagName && node.tagName == 'UL')
			return node
	return null
}

//----------------------------------------------------------------------
// Get current viewport coordinates & dimensions

function mpGetViewPort ()
{
	var vp = getViewPort()

	// Hack to allow for scroll bars that may appear
	// after dynamic menu placement

	vp.width -= 20
	vp.height -= 20

	return vp
}

//----------------------------------------------------------------------
// The world's most popular lousy browser
// Not working -- not used

function mpIEHack (menu)
{
	var ief

	if (!(ief = menu.mpIEFrame))
	{
		ief = menu.mpIEFrame = document.createElement('iframe')
		ief.setAttribute('frameborder', '0')
		ief.setAttribute('scrolling', 'no')
		Element.setStyle(ief, { 'position': 'absolute', 'zIndex': '600' })
		menu.parentNode.appendChild(ief)
	}

	ief.setAttribute('height', menu.offsetHeight)
	ief.setAttribute('width', menu.offsetWidth)
	Element.setStyle(ief, { 'display': '' })
	Position.clone(menu, ief)
	menu.parentNode.removeChild(menu)
	menu.style.top = menu.style.left = '0px'
	ief.document.innerHTML = ''
	ief.document.body.appendChild(menu)
	alert(ief.document.body.innerHTML)
}

//----------------------------------------------------------------------
// Mouse-out handler for menu items

function mpMenuItemOut (target)
{
	// Create custom un-hover function one time

	if (!target.mpUnhover)
		target.mpUnhover = function () { mpUnhover(target) }

	// Delay the un-hover long enough to see if this item will get
	// a new mouse-over as part of a transition to a sub-menu

	if (!target.mpUnhoverEvent)
		target.mpUnhoverEvent = setTimeout(target.mpUnhover, 50)
}

//----------------------------------------------------------------------
// Increasing z-index in IE for overlapping menus

var mpNextIEZ = 500

//----------------------------------------------------------------------
// Mouse-over handler for menu items

function mpMenuItemOver (target)
{
	if (target.mpUnhoverEvent)
	{
		// Cancel un-hover
		clearTimeout(target.mpUnhoverEvent)
		target.mpUnhoverEvent = null
	}

	// Simulate :hover with hover class for the benefit of IE < 7

	Element.addClassName(target, 'hover')

	if (document.all)	// Apply IE z-index hack
		target.style.zIndex = ++mpNextIEZ

	// Perform just-in-time setup and positioning of optional sub-menu

	if (target.mpSubMenu = mpSetup(target))
		mpPosition(target)
}

//----------------------------------------------------------------------
// Mouse-over handler for menus

function mpMenuOver (e, target)
{
	// Look for an LI below this target associated with this event
	var li = Event.element(e || window.event)
	while (li != target && li.tagName != 'LI')
		li = li.parentNode

	// If we found one without a mouse-over handler, we'll
	// have to call it ourselves this time

	if (li.tagName != 'LI' || li.mpReady)
		li = null

	// Perform mostly-in-time setup

	var ul = mpSetup(target)
	if (ul)			// maybe under e.g. DIV
	{
		ul.mpDirEl = target	// Element with direction classes
		mpSetup(ul)
		if (document.all)	// Apply IE z-index hack
			ul.style.zIndex = ++mpNextIEZ
	}
	else
	{
		if (Element.hasClassName(target.parentNode, 'menu'))
			target.mpDirEl = target.parentNode
		if (document.all)	// Apply IE z-index hack
			target.style.zIndex = ++mpNextIEZ
	}

	// Invoke mouse-over handler for LI that wasn't set up in time

	if (li)
		mpMenuItemOver(li)
}

//----------------------------------------------------------------------
// Position a UL sub-menu upon LI parent menu item hover

function mpPosition (item)
{
	var menu = mpGetParentMenu(item)
	var subMenu = item.mpSubMenu
	var vp = mpGetViewPort()

	// Make sure the sub-menu is visible before referencing its size

	var origDisp = subMenu.style.display
	subMenu.style.display = 'block'
	if (!menu.mpDirection)
		mpGetDirection(menu, item)

	// Don't position if the sub-menu won't fit in the viewport

	if (subMenu.offsetWidth > vp.width || subMenu.offsetHeight > vp.height)
	{
		// Revert to CSS visibility
		subMenu.style.display = origDisp
	  	return
	}

	// NB: Position.page is the position within the *viewport*,
	//     *NOT* the position within the document/entire web page!!

	var itemPos = Position.page(item)
	var info = {
		'item': item,
		'itemPos': {
			'x0': itemPos[0], 'y0': itemPos[1],
			'x1': itemPos[0] + item.offsetWidth,
			'y1': itemPos[1] + item.offsetHeight
		},
		'menu': menu,
		'subMenu': subMenu,
		'viewPort': vp
	}

	switch (menu.mpDirection[0])
	{
	case 'right':	// Fly out right (left if out of room)
		mpTryPosRight(info) || mpTryPosLeft(info)
		break
	case 'left':	// Fly out left (right if out of room)
		mpTryPosLeft(info) || mpTryPosRight(info)
		break
	case 'down':	// Drop down (raise up if out of room)
		mpTryPosDown(info) || mpTryPosUp(info)
		break
	case 'up':	// Raise up (drop down if out of room)
		mpTryPosUp(info) || mpTryPosDown(info)
		break
	}

	subMenu.style.display = origDisp	// Revert to CSS visibility
}

//----------------------------------------------------------------------
// Perform just-in-time menu setup, traversing child sub-trees to catch
// mouse-overs on next level ULs or LIs

function mpSetup (target)
{
	if (!target.childNodes || !target.childNodes.length)
		return null

	// Inner functions to create event handler closures

	function makeItemOut (node)
	{ return function () { mpMenuItemOut(node) } }

	function makeItemOver (node)
	{ return function () { mpMenuItemOver(node) } }

	function makeMenuOver (node)
	{ return function (e) { mpMenuOver(e, node) } }

	// Add appropriate mouse events and look for an LI sub-menu

	var subMenu
	for (var i = target.childNodes.length; --i >= 0; )
	{
		var next = target.childNodes[i]
		if (next.tagName && next.tagName == 'LI')
		{
			if (!next.mpReady)
			{
				next.mpReady = true
				Event.observe(next, 'mouseover', makeItemOver(next))
				Event.observe(next, 'mouseout', makeItemOut(next))
			}
		}
		else if (next.tagName && next.tagName == 'UL')
		{
			if (!next.mpReady)
			{
				next.mpReady = true
				Event.observe(next, 'mouseover', makeMenuOver(next))
			}
			return next
		}
		else if (subMenu = mpSetup(next))
			return subMenu
	}

	return null
}

//----------------------------------------------------------------------
// Try to position the sub-menu

function mpTryPos (info, pos)
{
	var item = info.item, itemPos = info.itemPos
	var subMenu = info.subMenu, vp = info.viewPort
	var xMinOver = (subMenu.offsetWidth < mpMinOverlap)?
	  subMenu.offsetWidth: mpMinOverlap
	var yMinOver = (subMenu.offsetHeight < mpMinOverlap)?
	  subMenu.offsetHeight: mpMinOverlap
	var adjustLeft = 0, adjustUp = 0

	// Initial sub-menu viewport coordinates before adjustments

	var subPos = { 'x0': itemPos.x0 + pos.x, 'y0': itemPos.y0 + pos.y }
	subPos.x1 = subPos.x0 + subMenu.offsetWidth
	subPos.y1 = subPos.y0 + subMenu.offsetHeight

	// Calculate any adjustments needed to make the sub-menu
	// fit into the viewport

	if (subPos.x0 < 0)		// Left side too far left
		adjustLeft = subPos.x0
	else if (subPos.x1 > vp.width)	// Right side too far right
		adjustLeft = subPos.x1 - vp.width

	if (subPos.y0 < 0)		// Top too high
		adjustUp = subPos.y0
	else if (subPos.y1 > vp.height)	// Bottom too low
		adjustUp = subPos.y1 - vp.height

	// Alter any adjustments as needed to keep minimum overlap

	if (pos.adjust & 1)
		if (subPos.x1 - adjustLeft < itemPos.x0 + xMinOver)
			adjustLeft = subPos.x1 - itemPos.x0 - xMinOver
		else if (subPos.x0 - adjustLeft > itemPos.x1 - xMinOver)
		  	adjustLeft = subPos.x0 - itemPos.x1 + xMinOver

	if (pos.adjust & 2)
		if (subPos.y1 - adjustUp < itemPos.y0 + yMinOver)
			adjustUp = subPos.y1 - itemPos.y0 - yMinOver
		else if (subPos.y0 - adjustUp > itemPos.y1 - yMinOver)
		  	adjustUp = subPos.y0 - itemPos.y1 + yMinOver

	// Fail this direction if a needed adjustment isn't allowed

	if ((adjustLeft && !(pos.adjust & 1)) ||
	  (adjustUp && !(pos.adjust & 2)))
	  	return 0

	// Dynamic positioning is done with left and top; make sure
	// there are no conflicts with bottom and right CSS positioning

	subMenu.style.bottom = subMenu.style.right = 'auto'

	// Now go ahead and position the sub-menu

	Position.clone(item, subMenu,
	  { 'setLeft': true, 'setTop': true,
	  'setWidth': false, 'setHeight': false,
	  'offsetLeft': pos.x - adjustLeft,
	  'offsetTop': pos.y - adjustUp })

	//if (document.all)
	//	mpIEHack(subMenu)

	// Direction to try first in the next sub-menu

	var direction = info.menu.mpDirection
	subMenu.mpDirection = [ pos.nextDir? pos.nextDir:
	  (direction[1]? direction[1]: direction[0]) ]

	return 1
}

//----------------------------------------------------------------------
// Try to position sub-menu below, adjusting left/right

function mpTryPosDown (info)
{
	return mpTryPos(info,
	  { 'x': 0, 'y': info.item.offsetHeight, 'adjust': 1 })
}

//----------------------------------------------------------------------
// Try to position sub-menu to the left, adjusting up/down

function mpTryPosLeft (info)
{
	return mpTryPos(info,
	  { 'x': -info.subMenu.offsetWidth, 'y': 0,
	  'adjust': 2, 'nextDir': 'left' })
}

//----------------------------------------------------------------------
// Try to position sub-menu to the right, adjusting up/down

function mpTryPosRight (info)
{
	return mpTryPos(info,
	  { 'x': info.item.offsetWidth, 'y': 0,
	  'adjust': 2, 'nextDir': 'right' })
}

//----------------------------------------------------------------------
// Try to position sub-menu above, adjusting left/right

function mpTryPosUp (info)
{
	return mpTryPos(info,
	  { 'x': 0, 'y': -info.subMenu.offsetHeight, 'adjust': 1 })
}

//----------------------------------------------------------------------
// Clear hover class emulation of :hover

function mpUnhover (target)
{
	//mpDebug('unhover ' + target)
	target.mpUnhoverEvent = null
	Element.removeClassName(target, 'hover')
}

// END
